/*
 * Read a range or all PCR registers of a TPM
 *
 * Copyright (C) 2011 Andreas Steffen
 *
 * HSR Hochschule fuer Technik Rapperswil
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <trousers/tss.h>

#define CKERR	if (result != TSS_SUCCESS) goto error

int main (int ac, char **av)
{
	TSS_HCONTEXT	hContext;
	TSS_HTPM	hTPM;
	UINT32		pcrStartIndex, pcrStopIndex, pcrMaxIndex, pcrIndex;
	UINT32		pcrValueLen, respDataLen, subCap;
	BYTE		*pcrValue, *respData;
	int			result, i;

	if (ac > 1 && (strcmp(av[1], "-h") == 0 || strcmp(av[1], "--help") == 0))
	{
		fprintf(stderr, "Usage: %s [startIndex] [stopIndex]\n", av[0]);
		exit (0);
	}

	/* Initialize TPM context */
	result = Tspi_Context_Create(&hContext); CKERR;
	result = Tspi_Context_Connect(hContext, NULL); CKERR;
	result = Tspi_Context_GetTpmObject (hContext, &hTPM); CKERR;

	/* Get the numbe of PCR registers */
	subCap = TSS_TPMCAP_PROP_PCR;
	result = Tspi_TPM_GetCapability(hTPM, TSS_TPMCAP_PROPERTY,
									4, (BYTE*)&subCap,
									&respDataLen, &respData); CKERR;
	if (respDataLen != 4)
	{
		fprintf(stderr, "Failure: "
						"number of PCR registers is not of type UINT32\n");
		exit(1);
	}
	pcrMaxIndex = *((UINT32*)respData) - 1;

	if (ac == 1)		/* read all PCR registers */
	{
		pcrStartIndex = 0;
		pcrStopIndex = pcrMaxIndex;
	}
	else
	{
		pcrStartIndex = atoi(av[1]);

		if (pcrStartIndex > pcrMaxIndex)
		{
			fprintf(stderr, "Error: startIndex > maxIndex of %u\n",
					pcrMaxIndex);
			exit(1);
		}

		if (ac == 2)	/* read a single PCR register */
		{
			pcrStopIndex = pcrStartIndex;
		}
		else			/* read a range of PCR registers */
		{
			pcrStopIndex = atoi(av[2]);
			if (pcrStartIndex > pcrStopIndex)
			{
				fprintf(stderr, "Error: startIndex > stopIndex\n");
				exit(1);
			}
			if (pcrStopIndex > pcrMaxIndex)
			{
				pcrStopIndex = pcrMaxIndex;
			}
		}
	}

	/* Read and output PCS values in HEX format */
	for (pcrIndex = pcrStartIndex; pcrIndex <= pcrStopIndex; pcrIndex++)
	{
		result = Tspi_TPM_PcrRead(hTPM, pcrIndex, &pcrValueLen, &pcrValue);
		printf("pcr%02u = ", pcrIndex);
		for (i = 0; i < pcrValueLen; i++)
		{
			printf("%02x", pcrValue[i]);
		}
		printf("\n");
		result = Tspi_Context_FreeMemory(hContext, pcrValue); CKERR;
	}
	exit(0);

error:
	fprintf (stderr, "Failure, error code: 0x%x\n", result);
	return 1;
}
